@@ -3,8 +3,8 @@
 ######### PSI-cd-hit written by Weizhong Li at http://cd-hit.org
 ################################################################################
 our $pid       = $$;
-our $db_in     = "";   ###################
-our $db_out    = "";   # input / output
+our $db_in;            ###################
+our $db_out;           # input / output
 our $len_t     = 10;   ###################
 our $NR_clstr  = 0.3;  #
 our $NR_clstre = -1;   #thresholds
@@ -15,18 +15,19 @@ our $circle    = 0;    #
 our $opt_g     = 1;    ####################
 our $blast_exe = "blastall -p blastp -m 8";               #########################
 our $prof_exe  = "blastpgp -m 8";                         #
-our $prof_para = "-j 3 -F F -e 0.001 -b 500 -v 500";      #
+our $prof_para = "-j 3 -F T -e 0.001 -b 500 -v 500";      #
 our $prof_db   = "";                                      #
-our $bl_para   = "-F F -e 0.000001 -b 100000 -v 100000";  #  program
+our $bl_para   = "-F T -e 0.000001 -b 100000 -v 100000";  #  program
 our $bl_STDIN  = 1;                                       #
 our $keep_bl   = 0;                                       #
 our $blast_prog= "blastp";                                #
 our $formatdb  = "formatdb";                              #########################
 our $exec_mode = "local";      #######################
-our $host_no   = 1;            #
-our $core_no   = 1;            # compute
+our $num_qsub   = 1;           #
+our $para_no   = 1;            # compute
 our $sh_file   = "";           #
-our $batch_no_per_node = 50;   #######################
+our $num_multi_seq = 50;       #
+our $batch_no_per_node = 100;  #######################
 our $reformat_seg = 50000;
 our $restart_seg  = 20000;
 our $job          = "";
@@ -39,11 +40,20 @@ our $db_log;
 our $db_out1;
 our $seq_dir;
 our $bl_dir;
+our $blm_dir;
 our $restart_file;
 our $tmp_db;
 our $remote_perl_script;
 our $remote_sh_script;
 our $bl_path;
+our $bl_plus = 1;   #### use blast+ 
+our $bl_threads = 1;
+our $skip_long = 0;
+our %qsub_ids = (); #### a list of qsub ids
+our %qstat_xml_data = ();
+our @blm8_buffer = ();
+our %blm8_data = ();
+
 
 sub parse_para_etc {
   my ($arg, $cmd);
@@ -60,18 +70,21 @@ sub parse_para_etc {
     elsif ($arg eq "-aS")         { $opt_aS    = shift; }
     elsif ($arg eq "-g")          { $opt_g     = shift; }
     elsif ($arg eq "-circle")     { $circle    = shift; }
+    elsif ($arg eq "-sl")         { $skip_long = shift; }
     ## program
     elsif ($arg eq "-prog")       { $blast_prog= shift; }
     elsif ($arg eq "-p")          { $prof_para = shift; }
-    elsif ($arg eq "-dprof")      { $prof_db   = shift; }
+    elsif ($arg eq "-dprof")      { $prof_db   = shift; die "option -dprof no longer supported!";}
     elsif ($arg eq "-s")          { $bl_para   = shift; }
     elsif ($arg eq "-k")          { $keep_bl   = shift; }
     elsif ($arg eq "-bs")         { $bl_STDIN  = shift; }
     ## compute
     elsif ($arg eq "-exec")       { $exec_mode = shift; }
-    elsif ($arg eq "-host")       { $host_no   = shift; }
-    elsif ($arg eq "-core")       { $core_no   = shift; }
+    elsif ($arg eq "-host")       { $num_qsub   = shift; }
+    elsif ($arg eq "-para")       { $para_no   = shift; }
     elsif ($arg eq "-shf")        { $sh_file   = shift; }
+    elsif ($arg eq "-blp")        { $bl_threads   = shift; }
+    elsif ($arg eq "-bat")        { $batch_no_per_node = shift; }
     ## job:
     elsif ($arg eq "-rs")         { $restart_seg = shift; }
     elsif ($arg eq "-rf")         { $reformat_seg= shift; }
@@ -81,9 +94,13 @@ sub parse_para_etc {
     elsif ($arg eq "-P")          { $bl_path   = shift;   }
     else                          { print_usage(); exit(); }
   }
+  if (not (defined($db_in) and defined($db_out))) {
+    print_usage(); exit();
+  }
 
   # speical jobs
-  if ($job eq "parse_blout") { job_parse_blout(); exit();}
+  if    ($job eq "parse_blout")       { job_parse_blout();       exit();}
+  elsif ($job eq "parse_blout_multi") { job_parse_blout_multi(); exit();}
 
   if ($blast_prog eq "blastn") {
     $formatdb  = "formatdb -p F";
@@ -95,7 +112,29 @@ sub parse_para_etc {
     $blast_exe    = "megablast -H 100 -D 2 -m 8";
   }
   elsif ($blast_prog eq "blastpgp") {
-    $blast_exe  = ($prof_db) ? "blastpgp -m 8" : "blastpgp -m 8 -j 3";
+    $blast_exe  = "blastpgp -m 8 -j 3";
+  }
+
+  #### for blast+
+  if ($bl_plus) {
+    $formatdb = "makeblastdb -dbtype prot -max_file_sz 8GB";
+    $blast_exe = "blastp -outfmt 6";
+    $bl_para   = "-seg yes -evalue 0.000001 -num_alignments 100000 -num_threads $bl_threads";  #  program
+
+    if ($blast_prog eq "blastn") {
+      $formatdb = "makeblastdb -dbtype nucl -max_file_sz 8GB";
+      $blast_exe    = "blastn -task blastn -outfmt 6";
+      $bl_para   = "-dust yes -evalue 0.000001 -num_alignments 100000 -num_threads $bl_threads";  #  program
+    }
+    elsif ($blast_prog eq "megablast") {
+      $blast_prog = "blastn"; #### back to blastn for blast parser type
+      $formatdb = "makeblastdb -dbtype nucl -max_file_sz 8GB";
+      $blast_exe    = "blastn -task megablast -outfmt 6";
+      $bl_para   = "-dust yes -evalue 0.000001 -num_alignments 100000 -num_threads $bl_threads";  #  program
+    }
+    elsif ($blast_prog eq "blastpgp") {
+      $blast_exe  = "psiblast -outfmt 6 -num_iterations 3 -num_threads $bl_threads";
+    }
   }
 
   if ($bl_path) {
@@ -111,13 +150,14 @@ sub parse_para_etc {
   $db_out1   = "$db_out.out";
   $seq_dir   = "$db_in-seq";
   $bl_dir    = "$db_in-bl";
+  $blm_dir   = "$db_in-blm";
   $restart_file   =" $db_out.restart";
 
   $tmp_db    = "$db_in.$pid";
   $remote_perl_script = "$tmp_db-bl.pl";
   $remote_sh_script   = "$tmp_db-bl.sh";
 
-  $cmd = `mkdir $bl_dir $seq_dir`;
+  $cmd = `mkdir $bl_dir $blm_dir $seq_dir`;
 
   write_remote_perl_script();
   write_remote_sh_script();
@@ -138,6 +178,7 @@ sub read_db {
       $seq =~ s/\s//g;
       if (length($seq) > $len_t) { add_seq($des, $seq); }
       $des = $ll; $seq = "";
+
     }
     else { $seq .= $ll; }
   }
@@ -155,6 +196,7 @@ sub read_db {
 
 sub add_seq {
   my ($des, $seq) = @_;
+  $des =~ s/\s.+$//;
   push(@seqs,   $seq);
   push(@dess,   $des);
   push(@lens,   length($seq));
@@ -180,6 +222,10 @@ sub open_LOG {
 }
 ########## END open_LOG
 
+sub write_LOG {
+  my $txt=shift;
+  print LOG "$txt\n";
+}
 
 {## use static variables
 my $last_NR90_no=0;
@@ -223,7 +269,7 @@ sub close_LOG {
 sub total_remote_cpu {
   my ($i, $j, $k, $ll);
   my $tt = 0;
-  for ($j=0; $j<$host_no; $j++) {
+  for ($j=0; $j<$num_qsub; $j++) {
     open(TCPU, "$seq_dir/host.$j.cpu") || next;
     while($ll = <TCPU>) {
       chop($ll);
@@ -235,6 +281,52 @@ sub total_remote_cpu {
 }
 ########## END total_remote_cpu
 
+#### process m8 format output from multi-query search
+sub job_parse_blout_multi{
+  my ($i, $j, $k, $tfh, $ll, $t1, $t2);
+
+  $tfh="BLM8";
+  open($tfh, $job_file) || die "can not open $job_file";
+
+  @blm8_buffer = ();
+  my $last_id = "";
+  my $this_id = "";
+  my $tquery;
+  while($ll = <$tfh>) {
+    next if ($ll =~ /^#/);
+    ($this_id, $t1) = split(/\s+/, $ll, 2);
+    
+    if (@blm8_buffer and ($this_id ne $last_id)) { #### blast results of last query
+      my @hits = process_blout_blastp_blastn();
+      $tquery = (split(/\./, $last_id))[0];
+      my $no1 = $#hits+1;
+      print ">$tquery\t$no1\n";
+      foreach $i (@hits) {
+        print join("\t", @{$i}), "\n";
+      }
+      print "#\n";
+      @blm8_buffer = ();
+    }
+    push(@blm8_buffer, $ll);
+    $last_id = $this_id;
+  }
+
+    if (@blm8_buffer and ($this_id ne $last_id)) { #### blast results of last query
+      my @hits = process_blout_blastp_blastn();
+      $tquery = (split(/\./, $last_id))[0];
+      my $no1 = $#hits+1;
+      print ">$tquery\t$no1\n";
+      foreach $i (@hits) {
+        print join("\t", @{$i}), "\n";
+      }
+      print "#\n";
+      @blm8_buffer = ();
+    }
+  close($tfh);
+  return;
+}
+########## END job_parse_blout_multi
+
 
 sub job_parse_blout {
   my ($i, $j, $k);
@@ -377,6 +469,45 @@ sub remove_raw_blout_bg {
 }
 ########## END remove_raw_blout_bg
 
+sub fish_other_homolog_multi {
+  my ($i, $j, $k, $i0, $j0, $k0);
+  $id = shift; # real idx, not sorted idx
+  my @hits = ();
+
+  if (defined($blm8_data{$id})) {
+    @hits = @{$blm8_data{$id}};
+  }
+
+  my $rep_len = $lens[$id];
+
+  foreach $i (@hits) {
+    my $id1 = $i->[0];
+    next unless ($id1 < $NR_no);
+    next if ($idens[$id1] eq "*"); #existing reps
+    next if ($lens[$id1] > $rep_len); # in opt_g=1 mode, preventing it from being clustered into short rep
+
+    if ( $passeds[$id1] ) { #### if this hit is better -g 1 mode
+      my $old_e = (split(/\//,$idens[$id1]))[0];
+      if ($i->[3] < $old_e) {
+        $idens[$id1]   = "$i->[3]/$i->[2]aa/$i->[1]%";
+        $passeds[$id1] = 1;
+        $NR_clstr_nos[$id1] = $NR90_no;
+      }
+      next;
+    }
+
+    $idens[$id1]   = "$i->[3]/$i->[2]aa/$i->[1]%";
+    $passeds[$id1] = 1;
+    $NR_clstr_nos[$id1] = $NR90_no;
+    $NR_passed++;
+  }
+  if (defined($blm8_data{$id})) {
+    delete $blm8_data{$id};
+  }
+  return;
+}
+########## END fish_other_homolog_multi
+
 
 sub fish_other_homolog {
   my ($i, $j, $k, $i0, $j0, $k0);
@@ -482,6 +613,7 @@ sub keep_top_hsp {
   for ($i=0; $i<$self->{no}; $i++) {
     my $p = $self->{sbj}->[$i];
     my ($id1, $len_sub) = split(/\./, $p->{id});
+    next unless ($len_sub >0) ;
     
     if (not defined($id_exist{$id1})) {
       $id_exist{$id1} = 1;
@@ -556,7 +688,7 @@ sub process_blout_blastp_blastn {
 
   #### need $len_rep
   my $len_rep = 0;
-  my $bl = readblast_m8("", $blout);
+  my $bl = defined($blout) ? readblast_m8("", $blout) : readblast_m8_buffer();
   if ($blast_prog eq "blastn") { keep_strand_with_top_hsp($bl); }
   if (($blast_prog eq "blastpgp") and (not $prof_db)) {keep_hsp_of_last_round($bl); }
 
@@ -569,6 +701,7 @@ sub process_blout_blastp_blastn {
       my $frame = $p->{frame};
       if (not $len_rep) {$len_rep = (split(/\./,$p->{qid}))[1]; }
       my $iden       = $p->{iden};
+      next unless (($len_sub >0) and ($len_rep>0));
       my $cov_aS     = $p->{alnln} / $len_sub;
       my $cov_aL     = $p->{alnln} / $len_rep;
       my $exp1       = $p->{expect};
@@ -592,6 +725,7 @@ sub process_blout_blastp_blastn {
       my ($id1, $len_sub) = split(/\./, $p->{id});
       my $frame = $p->{frame};
       if (not $len_rep) {$len_rep = (split(/\./,$p->{qid}))[1]; }
+      next unless (($len_sub >0) and ($len_rep>0));
 
       if ($hsp_no) {
         if ($id1 ne $hsp[0]->[0]) {
@@ -730,6 +864,49 @@ sub cross1_before_2013_0818 {
 }
 ########## END cross1
 
+sub readblast_m8_buffer {
+  my ($i, $j, $k, $ll, $no); 
+  my @this_sbj = ();
+  $no = 0;
+  while($ll = shift @blm8_buffer) {
+    chop($ll);
+    my @lls = split(/\t/,$ll);
+    my $frame = "";
+       $frame .= ($lls[6] < $lls[7]) ? "+" : "-";
+       $frame .= ($lls[8] < $lls[9]) ? "+" : "-";
+    next unless ($lls[0] and $lls[1]);
+    $this_sbj[$no] = {
+      'qid'     => $lls[0],
+      'id'      => $lls[1],
+      'iden'    => $lls[2],
+      'alnln'   => $lls[3],
+      'ms'      => $lls[4],
+      'gap'     => $lls[5],
+      'qfrom'   => $lls[6],
+      'qend'    => $lls[7],
+      'sfrom'   => $lls[8],
+      'send'    => $lls[9],
+      'expect'  => $lls[10],
+      'score'   => $lls[11],
+      'frame'   => $frame,
+    };
+
+    $no++;
+# BLASTP 2.2.24 [Aug-08-2010]
+# Query: gi|388328107|pdb|4DDG|A Chain A, Crystal Structure Of Human Otub1UBCH5B~UBUB
+# Database: pdbaa.fa
+# Fields: Query id, Subject id, % identity, alignment length, mismatches, gap openings, q. start, q. end, s. start, s. end, e-value, bit score
+#gi|388328107|pdb|4DDG|A gi|388328107|pdb|4DDG|A 91.81   171     9       3       6       171     1       171     6e-89    323
+#gi|388328107|pdb|4DDG|A gi|388328107|pdb|4DDG|A 96.51   86      3       0       235     320     155     240     2e-41    166
+  }
+  my $self = {
+    'no'  => $no,
+    'sbj' => [@this_sbj],
+  };
+  return $self;
+}
+########## END readblast_m8
+
 sub readblast_m8 {
   my ($i, $j, $k, $ll, $no); 
   my ($q_seq, $filename) = @_;
@@ -747,6 +924,7 @@ sub readblast_m8 {
     my $frame = "";
        $frame .= ($lls[6] < $lls[7]) ? "+" : "-";
        $frame .= ($lls[8] < $lls[9]) ? "+" : "-";
+    next unless ($lls[0] and $lls[1]);
     $this_sbj[$no] = {
       'qid'     => $lls[0],
       'id'      => $lls[1],
@@ -791,6 +969,7 @@ sub blast_formatdb {
   for ($i0=$NR_no-1; $i0>=0; $i0--) { ### from shortest to longest
     $i = $NR_idx[$i0];
     last if ($idens[$i] eq "*"); ### last if reach rep
+    next if ($lens[$i] < $opt_aL_lower_band);
     next if ($passeds[$i] and ($opt_g==0));
     my $seq = $seqs[$i];
     $seq =~ s/(.{70})/$1\n/g;
@@ -813,7 +992,10 @@ sub blast_formatdb {
 
   return(0, 0) unless ($j > 0);
 
-  my $cmd = `$formatdb -i $tmp_db`;
+  my $cmd_line = "$formatdb -i $tmp_db";
+     $cmd_line = "$formatdb -in $tmp_db" if ($bl_plus);
+  my $cmd = `$cmd_line`;
+     
   ((-e "$tmp_db.phr") and (-e "$tmp_db.pin") and (-e "$tmp_db.psq")) ||
   ((-e "$tmp_db.nhr") and (-e "$tmp_db.nin") and (-e "$tmp_db.nsq")) ||
   ((-e "$tmp_db.00.phr") and (-e "$tmp_db.00.pin") and (-e "$tmp_db.00.psq")) ||
@@ -842,15 +1024,15 @@ Options
   input/output:
     -i  in_dbname, required
     -o  out_dbname, required
-    -l  length_of_throw_away_sequences, default 10
+    -l  length_of_throw_away_sequences, default $len_t 
 
   thresholds:
-    -c  clustering threshold (sequence identity), default 0.3
-    -ce clustering threshold (blast expect), default -1, 
+    -c  clustering threshold (sequence identity), default $NR_clstr
+    -ce clustering threshold (blast expect), default $NR_clstre, 
         it means by default it doesn't use expect threshold, 
         but with positive value, the program cluster seqs if similarities
         meet either identity threshold or expect threshold 
-    -G  (1/0) use global identity? default 1
+    -G  (1/0) use global identity? default $g_iden
         two sequences Long (i.e. representative) and Short (redunant) may have multiple
         alignment fragments (i.e. HSPs), see:
         seq1  xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx   Long sequence
@@ -869,17 +1051,17 @@ Options
         Local identity  = identity of the top high score HSP
         if you prefer to use -G 0, it is suggested that you also
         use -aS, -aL, such as -aS 0.8, to prevent very short matches.
-    -aL	alignment coverage for the longer sequence, default 0.0
+    -aL	alignment coverage for the longer sequence, default $opt_aL
  	if set to 0.9, the alignment must covers 90% of the sequence
-    -aS	alignment coverage for the shorter sequence, default 0.0
+    -aS	alignment coverage for the shorter sequence, default $opt_aS
  	if set to 0.9, the alignment must covers 90% of the sequence
-    -g  (1/0), default 0
+    -g  (1/0), default $opt_g
         by cd-hit's default algorithm, a sequence is clustered to the first 
         cluster that meet the threshold (fast cluster). If set to 1, the program
         will cluster it into the most similar cluster that meet the threshold
         (accurate but slow mode)
         but either 1 or 0 won't change the representatives of final clusters
-    -circle (1/0), default 0
+    -circle (1/0), default $circle
         when set to 1, treat sequences as circular sequence.
         bacterial genomes, plasmids are circular, but their genome coordinate maybe arbitary, 
         the 2 HSPs below will be treated as non co-linear with -circle 0
@@ -887,55 +1069,78 @@ Options
               -------------circle-----------    
               |                            |   
         seq1  xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx      genome / plasmid 1 
-               \\\\\\\\      /////////////         
-                \\\\\\\\    /////////////                
+               \\\\\\\\\\\\\\\\      /////////////         
+                \\\\\\\\\\\\\\\\    /////////////                
                   HSP 2 -> ////HSP 1 ///   <-HSP 2
-                          /////////////     \\\\\\\\
-                         /////////////       \\\\\\\\
+                          /////////////     \\\\\\\\\\\\\\\\ 
+                         /////////////       \\\\\\\\\\\\\\\\ 
         seq2           xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx   genome / plasmid 2
                        |                             |
                        -----------circle--------------
+    -sl, length of very long sequences to be skipped, default $skip_long, 
+        e.g. -sl 5000 means sequences longer than 5000 aa will be treated as singleton clusters
+             without clustering, to save time, especially when there is -aL option in place, very
+             long sequences will not be clustered anyway.
+        -sl 0 means no skipping
    program:
-     -prog (blastp, blastn, megablast, blastpgp), default blastp 
-     -p  profile search para, default
-           "-j 3 -F F -e 0.001 -b 500 -v 500"
+     -prog (blastp, blastn, megablast, blastpgp), default $blast_prog
+     -p  profile search para, default 
+         "$prof_para"
      -dprof database for building PSSM, default using input
          you can also use another database that is more comprehensive like NR80
-     -s  blast search para, default
-           "-F F -e 0.000001 -b 100000 -v 100000"
-     -bs (1/0) default 1
+     -s  blast search para, default 
+         "$bl_para"
+     -bs (1/0) default $bl_STDIN
         pipe blast results from into parser instead of save in hard drive (save time)
 
    compute:
-     -exec (qsub, local) default local
+     -exec (qsub, local) default $exec_mode
           this program writes a shell script to run blast, this script is
           either performed locally by sh or remotely by qsub
           with qsub, you can use PBS, SGE etc
-     -host number of hosts for qsub 
-     -core number of cpu cores per computer, default 1
+     -host number of qsub jobs, default $num_qsub
+     -para number of parallel blast job per qsub job (each blast can use multi cores), default $para_no
+           one qsub script can run multiple blast jobs
+     -blp  number of threads per blast job, default $bl_threads
+           number of threads per blast job (option -blp) X number of parallel blast job per qsub job (option -para)
+           should <= the number of cores in your computer
+           if your computer grid has 32 cores / node, do either of the followings
+           -para 4  -blp 8
+           -para 8  -blp 4 preferred
+           -para 16 -blp 2
+           -para 32 -blp 1
+     -bat number of sequences a blast job to process, $batch_no_per_node
      -shf a filename for add local settings into the job shell script
           for example, when you run PBS jobs, you can add quene name etc in this
           file and this script will add them into the job shell script
-e.g. your file may have followings 
+e.g. template file for PBS
+#!/bin/sh
 #PBS -v PATH
 #PBS -l walltime=8:00:00
-#PBS -q jobqueue
+#PBS -q job_queue.q
+
+e.g. template file for SGE or OGE
+#!/bin/sh
+#\$ -v PATH
+#\$ -q job_queue.q 
+#\$ -V
+#\$ -pe orte 8
 
   job:
-    -rs steps of save restart file and clustering output, default 5000
+    -rs steps of save restart file and clustering output, default $restart_seg
         everytime after process 5000 sequences, program write a 
         restart file and current clustering information
     -restart restart file, readin a restart file
         if program crash, stoped, termitated, you can restart it by
         add a option "-restart sth.restart"
-    -rf steps of re format blast database, default 200,000
+    -rf steps of re format blast database, default $reformat_seg
         if program clustered 200,000 seqs, it remove them from seq
         pool, and re format blast db to save time
     -J  job, job_file, exe specific jobs like parse blast outonly
-        DON'T use it, it is only used by this program itself
+        DO NOT use it, it is only used by this program itself
     -k (1/0) keep blast raw output file, default $keep_bl
 
-    -P path to executables
+    -P path to blast executables
 EOD
 
 
@@ -958,18 +1163,144 @@ EOD
 ########## END print_usage
 
 
-## like above, but don't assign seqs to specific node
-## while let nodes run them autoly
-sub  run_batch_blast3 {
+## copied from run_batch_blast3 
+## run multi seq per sample
+## wait for all jobs to finish
+sub run_batch_blast3_multi {
   my $i0 = shift;
-  my ($id, $i, $j, $k);
+  my ($id, $i, $j, $k, $cmd, $ll);
  
-  my $total_jobs = $batch_no_per_node * $host_no * $core_no;
+  my $total_jobs = $batch_no_per_node * $num_qsub * $para_no;
 
   for ($k=0; $i0<$NR_no; $i0++) {
     $id = $NR_idx[$i0];
     next if ($passeds[$id]);
     next if ($in_bg[$id]);
+    next if ($lens[$id] < $opt_aL_upper_band);
+    $in_bg[$id] = 1;
+
+    my $seq = $seqs[$id];
+    
+    if (($k % $num_multi_seq) ==0) { #### reopen
+      close(SEQ) if ($k > 0);
+      open(SEQ, "> $seq_dir/$id") || die "Can not write";
+    }
+    #print SEQ "$dess[$id]\n$seq\n";
+    print SEQ ">$id.$lens[$id]\n$seq\n";
+    $k++;
+    last if ($k >= $total_jobs);
+  }
+  close(SEQ);
+
+  if ($exec_mode eq "qsub") {
+    for ($j=0; $j<$num_qsub; $j++) {
+      my $t = "psi-cd-hit-$j";
+      my $cmd = `qsub -N $t $remote_sh_script $j`; #### pass $j to qsub command
+      my $qsub_id = 0;
+      if ($cmd =~ /(\d+)/) { $qsub_id = $1;} else {die "can not submit qsub job and return a id\n";}
+      print LOG "qsub querying $j, PID $qsub_id\n";
+      $qsub_ids{$qsub_id} = 1;
+    }
+  }
+  elsif ($exec_mode eq "local") {
+    #my $cmd = `sh $remote_sh_script >/dev/null 2>&1 &`;
+    my $cmd = `sh $remote_sh_script`;
+  }
+
+  #### wait finish all submitted 
+  if ($exec_mode eq "qsub") {
+    while(1) {
+      SGE_qstat_xml_query();
+      last unless (%qsub_ids);
+
+      my $wait_flag = 0;
+      foreach my $qsub_id (keys %qsub_ids) {
+        if (defined($qstat_xml_data{$qsub_id})) { #### still running
+          $wait_flag = 1;
+        } 
+        else {
+          delete $qsub_ids{$qsub_id};
+        }
+      }
+
+      if ($wait_flag) {print LOG "wait submitted jobs\n"; sleep(1); }
+    }
+  }
+
+  #### read in all parsed blast output
+  %blm8_data =();
+  opendir(BLMDIR, $blm_dir) || die "can not open $blm_dir";
+  my @bl_files = grep { /^\d/ } readdir(BLMDIR);
+  closedir(BLMDIR);
+
+  foreach my $blf (@bl_files) {
+    open(BLMTMP, "$blm_dir/$blf") || next;
+    while($ll = <BLMTMP>) {
+      next if ($ll =~ /^#/);
+      chop($ll);
+      if ($ll =~ /^>/) {
+
+        my ($id, $no1) = split(/\s+/, substr($ll,1));
+        my @hits = ();
+        for ($j=0; $j<$no1; $j++) {
+          $ll=<BLMTMP>; chop($ll);
+          push(@hits, [split(/\t/,$ll)]);
+        }
+        if ($no1>=1) {
+          $blm8_data{$id} = [@hits];
+        }
+      }
+    }
+    close(BLMTMP);
+
+    $cmd = `rm -f $blm_dir/$blf`;
+    print LOG "parse and then rm $blm_dir/$blf\n";
+  }
+  return;
+}
+
+sub  run_batch_blast3 {
+  my $i0 = shift;
+  my ($id, $i, $j, $k, $cmd);
+ 
+  #### wait before qsubs
+  if ($exec_mode eq "qsub") {
+    while(1) {
+      SGE_qstat_xml_query();
+      last unless (%qsub_ids);
+
+      my $wait_flag = 0;
+      foreach my $qsub_id (keys %qsub_ids) {
+        if (defined($qstat_xml_data{$qsub_id})) { #### still running
+          $wait_flag = 1;
+          $cmd = `qdel -f $qsub_id`; #### at this point, all running jobs are not necessary,
+          print LOG "force delete un necessary job $qsub_id\n";
+        } 
+        else {
+          delete $qsub_ids{$qsub_id};
+        }
+      }
+
+      if ($wait_flag) {print LOG "wait submitted jobs\n"; sleep(1); }
+    }
+
+    #### delete seq files from last batch
+    opendir(DIR1, $seq_dir);
+    my @files = grep { /^\d/ } readdir(DIR1);
+    closedir(DIR1);
+    foreach $i (@files) {
+      $cmd = `rm -f $seq_dir/$i`;
+      print LOG "remove un necessary seq file $i\n"
+    }
+  }
+
+  my $total_jobs = $batch_no_per_node * $num_qsub * $para_no;
+
+  for ($k=0; $i0<$NR_no; $i0++) {
+    $id = $NR_idx[$i0];
+    next if ($passeds[$id]);
+    next if ($in_bg[$id]);
+    next if ($lens[$id] < $opt_aL_upper_band);
     $in_bg[$id] = 1;
 
     my $seq = $seqs[$id];
@@ -982,14 +1313,18 @@ sub  run_batch_blast3 {
   }
 
   if ($exec_mode eq "qsub") {
-    for ($j=0; $j<$host_no; $j++) {
+    for ($j=0; $j<$num_qsub; $j++) {
       my $t = "psi-cd-hit-$j";
-      print LOG "PBS querying $j\n";
       my $cmd = `qsub -N $t $remote_sh_script`;
+      my $qsub_id = 0;
+      if ($cmd =~ /(\d+)/) { $qsub_id = $1;} else {die "can not submit qsub job and return a id\n";}
+      print LOG "qsub querying $j, PID $qsub_id\n";
+      $qsub_ids{$qsub_id} = 1;
     }
   }
   elsif ($exec_mode eq "local") {
-    my $cmd = `sh $remote_sh_script >/dev/null 2>&1 &`;
+    #my $cmd = `sh $remote_sh_script >/dev/null 2>&1 &`;
+    my $cmd = `sh $remote_sh_script`;
   }
 
   return;
@@ -999,24 +1334,26 @@ sub  run_batch_blast3 {
 
 sub write_remote_sh_script {
   my ($i, $j, $k);
-  my $local_sh = "";
+  my $local_sh = <<EOD;
+#!/bin/sh
+#PBS -v PATH
+#\$ -v PATH
+EOD
+
   if ($sh_file) {
     $local_sh = `cat $sh_file`;
   }
 
   open(RESH, "> $remote_sh_script") || die;
   print RESH <<EOD;
-#!/bin/bash
-#\$ -S /bin/bash
-#\$ -v PATH
-#PBS -v PATH
 $local_sh
 
+para=\$1
 cd $pwd
 EOD
 
-  for ($k=0; $k<$core_no; $k++){
-    print RESH "./$remote_perl_script $k&\n"
+  for ($k=0; $k<$para_no; $k++){
+    print RESH "./$remote_perl_script $k \$para &\n"
   }
   print RESH "wait\n\n";
 
@@ -1027,16 +1364,17 @@ EOD
 
 sub write_remote_perl_script {
   my $dir1 = ".";
-  my $bl2  = ($prof_db) ?
-     "$blast_exe -d $dir1/$tmp_db $bl_para -R $bl_dir/\$id.prof":
-     "$blast_exe -d $dir1/$tmp_db $bl_para";
-  my $cc = ($prof_db) ? 1 : 0;
-  if ($prof_db) { my $cmd=`formatdb -i $prof_db`; }
+  my $bl2  = "$blast_exe -d $dir1/$tmp_db $bl_para";
+     $bl2  = "$blast_exe -db $dir1/$tmp_db $bl_para" if ($bl_plus);
+
+  my $opti = "-i"; $opti = "-query" if ($bl_plus);
+  my $opto = "-o"; $opto = "-out"   if ($bl_plus);
 
   open(REPERL, "> $remote_perl_script") || die;
   print REPERL <<EOD;
 #!/usr/bin/perl
 \$host = shift;
+\$instance = shift;
 \$arg = shift;
 
 #### random sleep, rand() can be a fraction of second
@@ -1063,25 +1401,24 @@ foreach \$id (\@ids) {
   next if (-e "$seq_dir/\$id.lock");
   \$cmd = `touch $seq_dir/\$id.lock`;
 
-  if ($cc) {
-    \$cmd = `$prof_exe -d $prof_db $prof_para -i $seq_dir/\$id -C $bl_dir/\$id.prof`;
+  if ($num_multi_seq) {
+    \$cmd = `$bl2 $opti $seq_dir/\$id $opto $bl_dir/\$id`;
+    \$cmd =                         `$script_name -J parse_blout_multi $bl_dir/\$id -c $NR_clstr -ce $NR_clstre -aS $opt_aS -aL $opt_aL -G $g_iden -prog $blast_prog -bs 0 >> $blm_dir/\$host.\$instance`;
   }
-
-  if ($bl_STDIN) {
-    \$cmd = `$bl2 -i $seq_dir/\$id | $script_name -J parse_blout $bl_dir/\$id -c $NR_clstr -ce $NR_clstre -aS $opt_aS -aL $opt_aL -G $g_iden -prog $blast_prog -bs 1`;
+  elsif ($bl_STDIN) {
+    \$cmd = `$bl2 $opti $seq_dir/\$id | $script_name -J parse_blout $bl_dir/\$id -c $NR_clstr -ce $NR_clstre -aS $opt_aS -aL $opt_aL -G $g_iden -prog $blast_prog -bs 1`;
   }
   else {
-    \$cmd = `$bl2 -i $seq_dir/\$id -o $bl_dir/\$id`;
+    \$cmd = `$bl2 $opti $seq_dir/\$id $opto $bl_dir/\$id`;
     \$cmd =                         `$script_name -J parse_blout $bl_dir/\$id -c $NR_clstr -ce $NR_clstre -aS $opt_aS -aL $opt_aL -G $g_iden -prog $blast_prog -bs 0`;
   }
   \$cmd = `rm -f  $seq_dir/\$id`;
   \$cmd = `rm -f  $seq_dir/\$id.lock`;
-  if ($cc) { \$cmd = `rm -f  $bl_dir/\$id.prof`; }
 }
 
 (\$tu, \$ts, \$cu, \$cs) = times();
 \$tt = \$tu + \$ts + \$cu + \$cs;
-\$cmd = `echo \$tt >> $seq_dir/host.\$host.cpu`;
+\$cmd = `echo \$tt >> $seq_dir/host.\$host.\$instance.cpu`;
 
 EOD
   close(REPERL);
@@ -1111,6 +1448,77 @@ sub wait_blast_out {
 ########## END wait_blast_out
 
 
+sub SGE_qstat_xml_query {
+  my ($i, $j, $k, $cmd, $ll);
+  %qstat_xml_data = (); #### global
+  $cmd = `qstat -f -xml`;
+  if ($cmd =~ /<queue_info/) { #### dummy 
+    $qstat_xml_data{"NULL"}= ["NULL","NULL"];
+  }
+  my $tmp = <<EOD;
+<?xml version='1.0'?>
+<job_info  xmlns:xsd="http://gridscheduler.svn.sourceforge.net/viewvc/gridscheduler/trunk/source/dist/util/resources/schemas/qstat/qstat.xsd?revision=11">
+  <queue_info>
+    <Queue-List>
+      <name>all.q\@master</name>
+      <qtype>BIP</qtype>
+      <slots_used>0</slots_used>
+      <slots_resv>0</slots_resv>
+      <slots_total>0</slots_total>
+      <load_avg>0.08000</load_avg>
+      <arch>linux-x64</arch>
+    </Queue-List>
+...
+    <Queue-List>
+      <name>all.q\@node016</name>
+      <qtype>BIP</qtype>
+      <slots_used>32</slots_used>
+      <slots_resv>0</slots_resv>
+      <slots_total>32</slots_total>
+      <load_avg>42.59000</load_avg>
+      <arch>linux-x64</arch>
+      <job_list state="running"> ####### running jobs in this section
+        <JB_job_number>3535</JB_job_number>
+        <JAT_prio>0.51468</JAT_prio>
+        <JB_name>cd-hit</JB_name>
+        <JB_owner>ubuntu</JB_owner>
+        <state>r</state>
+        <slots>4</slots>
+      </job_list>
+...
+  </queue_info>
+  <job_info>
+    <job_list state="pending">  ######## pending jobs in this section
+      <JB_job_number>3784</JB_job_number>
+      <JAT_prio>0.60500</JAT_prio>
+      <JB_name>cd-hit</JB_name>
+      <JB_owner>ubuntu</JB_owner>
+      <state>qw</state>
+      <slots>32</slots>
+    </job_list>
+...
+  </job_info>
+</job_info>
+
+EOD
+  my @lls = split(/\n/, $cmd);
+  $i = 2; #### skip first 2 lines
+  for (;     $i<$#lls+1; $i++) {
+    if ($lls[$i] =~ /<job_list/) {
+      my ($id, $name, $state);
+      for (; $i<$#lls+1; $i++) {
+        last if ($lls[$i] =~ /<\/job_list/);
+        if ($lls[$i] =~ /<JB_job_number>(\d+)/) {  $id = $1;}
+        if ($lls[$i] =~ /<JB_name>([^<]+)/) { $name = $1;}
+        if ($lls[$i] =~ /<state>([^<]+)/) {$state = $1;}
+      }
+      if (defined($id) and defined($name) and defined($state)) {
+        $qstat_xml_data{$id} = [$name, $state];
+      }
+    }
+  }
+}
+
 
 1;
 
